@@ -1,5 +1,5 @@
 /* BFD back-end for Intel 386 COFF files.
-   Copyright 1990, 1991, 1992, 1993, 1994 Free Software Foundation, Inc.
+   Copyright 1990, 91, 92, 93, 94, 95, 96, 1997 Free Software Foundation, Inc.
    Written by Cygnus Support.
 
 This file is part of BFD, the Binary File Descriptor library.
@@ -16,23 +16,34 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 #include "bfd.h"
 #include "sysdep.h"
 #include "libbfd.h"
-#include "obstack.h"
+
 #include "coff/i386.h"
+
 #include "coff/internal.h"
+
+#ifdef COFF_WITH_PE
+#include "coff/pe.h"
+#endif
+
 #include "libcoff.h"
 
-static bfd_reloc_status_type coff_i386_reloc PARAMS ((bfd *abfd,
-						      arelent *reloc_entry,
-						      asymbol *symbol,
-						      PTR data,
-						      asection *input_section,
-						      bfd *output_bfd,
-						      char **error_message));
+static bfd_reloc_status_type coff_i386_reloc 
+  PARAMS ((bfd *, arelent *, asymbol *, PTR, asection *, bfd *, char **));
+static reloc_howto_type *coff_i386_rtype_to_howto
+  PARAMS ((bfd *, asection *, struct internal_reloc *,
+	   struct coff_link_hash_entry *, struct internal_syment *,
+
+	   bfd_vma *));
+
+#define COFF_DEFAULT_SECTION_ALIGNMENT_POWER (2)
+/* The page size is a guess based on ELF.  */
+
+#define COFF_PAGE_SIZE 0x1000
 
 /* For some reason when using i386 COFF the value stored in the .text
    section for a reference to a common symbol is the value itself plus
@@ -59,6 +70,7 @@ coff_i386_reloc (abfd, reloc_entry, symbol, data, input_section, output_bfd,
   if (output_bfd == (bfd *) NULL)
     return bfd_reloc_continue;
 
+
   if (bfd_is_com_section (symbol->section))
     {
       /* We are relocating a common symbol.  The current value in the
@@ -83,49 +95,74 @@ coff_i386_reloc (abfd, reloc_entry, symbol, data, input_section, output_bfd,
       diff = reloc_entry->addend;
     }
 
+
+#ifdef COFF_WITH_PE
+  if (reloc_entry->howto->type == 7)
+    {
+/*      diff -= coff_data(output_bfd)->link_info->pe_info.image_base.value;*/
+      exit(1);
+    }
+#endif
+
 #define DOIT(x) \
   x = ((x & ~howto->dst_mask) | (((x & howto->src_mask) + diff) & howto->dst_mask))
 
-  if (diff != 0)
-    {
-      const reloc_howto_type *howto = reloc_entry->howto;
-      unsigned char *addr = (unsigned char *) data + reloc_entry->address;
+    if (diff != 0)
+      {
+	reloc_howto_type *howto = reloc_entry->howto;
+	unsigned char *addr = (unsigned char *) data + reloc_entry->address;
 
-      switch (howto->size)
-	{
-	case 0:
+	switch (howto->size)
 	  {
-	    char x = bfd_get_8 (abfd, addr);
-	    DOIT (x);
-	    bfd_put_8 (abfd, x, addr);
-	  }
-	  break;
+	  case 0:
+	    {
+	      char x = bfd_get_8 (abfd, addr);
+	      DOIT (x);
+	      bfd_put_8 (abfd, x, addr);
+	    }
+	    break;
 
-	case 1:
-	  {
-	    short x = bfd_get_16 (abfd, addr);
-	    DOIT (x);
-	    bfd_put_16 (abfd, x, addr);
-	  }
-	  break;
+	  case 1:
+	    {
+	      short x = bfd_get_16 (abfd, addr);
+	      DOIT (x);
+	      bfd_put_16 (abfd, x, addr);
+	    }
+	    break;
 
-	case 2:
-	  {
-	    long x = bfd_get_32 (abfd, addr);
-	    DOIT (x);
-	    bfd_put_32 (abfd, x, addr);
-	  }
-	  break;
+	  case 2:
+	    {
+	      long x = bfd_get_32 (abfd, addr);
+	      DOIT (x);
+	      bfd_put_32 (abfd, x, addr);
+	    }
+	    break;
 
-	default:
-	  abort ();
-	}
-    }
+	  default:
+	    abort ();
+	  }
+      }
 
   /* Now let bfd_perform_relocation finish everything up.  */
   return bfd_reloc_continue;
 }
 
+#ifdef COFF_WITH_PE
+/* Return true if this relocation should
+   appear in the output .reloc section. */
+
+static boolean in_reloc_p(abfd, howto)
+     bfd * abfd;
+     reloc_howto_type *howto;
+{
+  return ! howto->pc_relative && howto->type != R_IMAGEBASE;
+}     
+#endif
+
+#ifndef PCRELOFFSET
+#define PCRELOFFSET false
+#endif
+
 static reloc_howto_type howto_table[] = 
 {
   {0},
@@ -146,8 +183,21 @@ static reloc_howto_type howto_table[] =
 	 true,	                /* partial_inplace */                      
 	 0xffffffff,            /* src_mask */                             
 	 0xffffffff,            /* dst_mask */                             
+	 true),                /* pcrel_offset */
+  /* {7}, */
+  HOWTO (R_IMAGEBASE,            /* type */                                 
+	 0,	                /* rightshift */                           
+	 2,	                /* size (0 = byte, 1 = short, 2 = long) */ 
+	 32,	                /* bitsize */                   
+	 false,	                /* pc_relative */                          
+	 0,	                /* bitpos */                               
+	 complain_overflow_bitfield, /* complain_on_overflow */
+	 coff_i386_reloc,       /* special_function */                     
+	 "rva32",	           /* name */                                 
+	 true,	                /* partial_inplace */                      
+	 0xffffffff,            /* src_mask */                             
+	 0xffffffff,            /* dst_mask */                             
 	 false),                /* pcrel_offset */
-  {7},
   {010},
   {011},
   {012},
@@ -167,7 +217,7 @@ static reloc_howto_type howto_table[] =
 	 true,			/* partial_inplace */                      
 	 0x000000ff,		/* src_mask */                             
 	 0x000000ff,		/* dst_mask */                             
-	 false),		/* pcrel_offset */
+	 PCRELOFFSET),		/* pcrel_offset */
   HOWTO (R_RELWORD,		/* type */                                 
 	 0,			/* rightshift */                           
 	 1,			/* size (0 = byte, 1 = short, 2 = long) */ 
@@ -180,7 +230,7 @@ static reloc_howto_type howto_table[] =
 	 true,			/* partial_inplace */                      
 	 0x0000ffff,		/* src_mask */                             
 	 0x0000ffff,		/* dst_mask */                             
-	 false),		/* pcrel_offset */
+	 PCRELOFFSET),		/* pcrel_offset */
   HOWTO (R_RELLONG,		/* type */                                 
 	 0,			/* rightshift */                           
 	 2,			/* size (0 = byte, 1 = short, 2 = long) */ 
@@ -193,7 +243,7 @@ static reloc_howto_type howto_table[] =
 	 true,			/* partial_inplace */                      
 	 0xffffffff,		/* src_mask */                             
 	 0xffffffff,		/* dst_mask */                             
-	 false),		/* pcrel_offset */
+	 PCRELOFFSET),		/* pcrel_offset */
   HOWTO (R_PCRBYTE,		/* type */                                 
 	 0,			/* rightshift */                           
 	 0,			/* size (0 = byte, 1 = short, 2 = long) */ 
@@ -206,7 +256,7 @@ static reloc_howto_type howto_table[] =
 	 true,			/* partial_inplace */                      
 	 0x000000ff,		/* src_mask */                             
 	 0x000000ff,		/* dst_mask */                             
-	 false),		/* pcrel_offset */
+	 PCRELOFFSET),		/* pcrel_offset */
   HOWTO (R_PCRWORD,		/* type */                                 
 	 0,			/* rightshift */                           
 	 1,			/* size (0 = byte, 1 = short, 2 = long) */ 
@@ -219,7 +269,7 @@ static reloc_howto_type howto_table[] =
 	 true,			/* partial_inplace */                      
 	 0x0000ffff,		/* src_mask */                             
 	 0x0000ffff,		/* dst_mask */                             
-	 false),		/* pcrel_offset */
+	 PCRELOFFSET),		/* pcrel_offset */
   HOWTO (R_PCRLONG,		/* type */                                 
 	 0,			/* rightshift */                           
 	 2,			/* size (0 = byte, 1 = short, 2 = long) */ 
@@ -232,7 +282,7 @@ static reloc_howto_type howto_table[] =
 	 true,			/* partial_inplace */                      
 	 0xffffffff,		/* src_mask */                             
 	 0xffffffff,		/* dst_mask */                             
-	 false)			/* pcrel_offset */
+	 PCRELOFFSET)		/* pcrel_offset */
 };
 
 /* Turn a howto into a reloc  nunmber */
@@ -242,12 +292,7 @@ static reloc_howto_type howto_table[] =
 #define I386 1			/* Customize coffcode.h */
 
 #define RTYPE2HOWTO(cache_ptr, dst) \
-	    cache_ptr->howto = howto_table + (dst)->r_type;
-
-/* On SCO Unix 3.2.2 the native assembler generates two .data
-   sections.  We handle that by renaming the second one to .data2.  It
-   does no harm to do this for any 386 COFF target.  */
-#define TWO_DATA_SECS
+	    (cache_ptr)->howto = howto_table + (dst)->r_type;
 
 /* For 386 COFF a STYP_NOLOAD | STYP_BSS section is part of a shared
    library.  On some other COFF targets STYP_BSS is normally
@@ -287,16 +332,109 @@ static reloc_howto_type howto_table[] =
       cache_ptr->addend += asect->vma;				\
   }
 
+/* We use the special COFF backend linker.  */
+#define coff_relocate_section _bfd_coff_generic_relocate_section
+
+static reloc_howto_type *
+coff_i386_rtype_to_howto (abfd, sec, rel, h, sym, addendp)
+     bfd *abfd;
+     asection *sec;
+     struct internal_reloc *rel;
+     struct coff_link_hash_entry *h;
+     struct internal_syment *sym;
+     bfd_vma *addendp;
+{
+
+  reloc_howto_type *howto;
+
+  howto = howto_table + rel->r_type;
+
+#ifdef COFF_WITH_PE
+  *addendp = 0;
+#endif
+
+  if (howto->pc_relative)
+    *addendp += sec->vma;
+
+  if (sym != NULL && sym->n_scnum == 0 && sym->n_value != 0)
+    {
+      /* This is a common symbol.  The section contents include the
+	 size (sym->n_value) as an addend.  The relocate_section
+	 function will be adding in the final value of the symbol.  We
+	 need to subtract out the current size in order to get the
+	 correct result.  */
+ 
+      BFD_ASSERT (h != NULL);
+
+
+#ifndef COFF_WITH_PE
+      /* I think we *do* want to bypass this.  If we don't, I have seen some data
+	 parameters get the wrong relcation address.  If I link two versions
+	 with and without this section bypassed and then do a binary comparison,
+	 the addresses which are different can be looked up in the map.  The 
+	 case in which this section has been bypassed has addresses which correspond
+	 to values I can find in the map */
+      *addendp -= sym->n_value;
+#endif
+    }
+
+  /* If the output symbol is common (in which case this must be a
+     relocateable link), we need to add in the final size of the
+     common symbol.  */
+  if (h != NULL && h->root.type == bfd_link_hash_common) 
+    *addendp += h->root.u.c.size;
+
+
+#ifdef COFF_WITH_PE
+  if (howto->pc_relative)
+    *addendp -= 4;
+
+  if (rel->r_type == R_IMAGEBASE)
+    {
+      *addendp -= pe_data(sec->output_section->owner)->pe_opthdr.ImageBase;
+    }
+#endif
+
+  return howto;
+}
+
+
+#define coff_bfd_reloc_type_lookup coff_i386_reloc_type_lookup
+
+
+static reloc_howto_type *
+coff_i386_reloc_type_lookup (abfd, code)
+     bfd *abfd;
+     bfd_reloc_code_real_type code;
+{
+  switch (code)
+    {
+    case BFD_RELOC_RVA:
+      return howto_table +R_IMAGEBASE;
+    case BFD_RELOC_32:
+      return howto_table + R_DIR32;
+    case BFD_RELOC_32_PCREL:
+      return howto_table + R_PCRLONG;
+    default:
+      BFD_FAIL ();
+      return 0;
+    }
+}
+
+
+
+#define coff_rtype_to_howto coff_i386_rtype_to_howto
+
 #include "coffcode.h"
 
-static bfd_target *
+static const bfd_target *
 i3coff_object_p(a)
      bfd *a;
 {
   return coff_object_p(a);
 }
 
-bfd_target
+const bfd_target
 #ifdef TARGET_SYM
   TARGET_SYM =
 #else
@@ -309,19 +447,28 @@ bfd_target
   "coff-i386",			/* name */
 #endif
   bfd_target_coff_flavour,
-  false,			/* data byte order is little */
-  false,			/* header byte order is little */
+  BFD_ENDIAN_LITTLE,		/* data byte order is little */
+  BFD_ENDIAN_LITTLE,		/* header byte order is little */
 
   (HAS_RELOC | EXEC_P |		/* object flags */
    HAS_LINENO | HAS_DEBUG |
-   HAS_SYMS | HAS_LOCALS | WP_TEXT),
+   HAS_SYMS | HAS_LOCALS | WP_TEXT | D_PAGED),
 
+#ifndef COFF_WITH_PE
   (SEC_HAS_CONTENTS | SEC_ALLOC | SEC_LOAD | SEC_RELOC), /* section flags */
+#else
+  (SEC_HAS_CONTENTS | SEC_ALLOC | SEC_LOAD | SEC_RELOC /* section flags */
+   | SEC_LINK_ONCE | SEC_LINK_DUPLICATES),
+#endif
+
+#ifdef TARGET_UNDERSCORE
+  TARGET_UNDERSCORE,		/* leading underscore */
+#else
   0,				/* leading underscore */
+#endif
   '/',				/* ar_pad_char */
   15,				/* ar_max_namelen */
 
-  2,				/* minimum alignment power */
   bfd_getl64, bfd_getl_signed_64, bfd_putl64,
      bfd_getl32, bfd_getl_signed_32, bfd_putl32,
      bfd_getl16, bfd_getl_signed_16, bfd_putl16, /* data */
