@@ -1,5 +1,5 @@
 /* BFD back-end data structures for a.out (and similar) files.
-   Copyright 1990, 91, 92, 93, 94, 95, 1996 Free Software Foundation, Inc.
+   Copyright 1990, 91, 92, 93, 94, 95, 96, 1997 Free Software Foundation, Inc.
    Written by Cygnus Support.
 
 This file is part of BFD, the Binary File Descriptor library.
@@ -105,6 +105,12 @@ struct aout_backend_data
      If not, the text section starts on the next page.  */
   unsigned char text_includes_header;
 
+  /* If this flag is set, then if the entry address is not in the
+     first SEGMENT_SIZE bytes of the text section, it is taken to be
+     the address of the start of the text section.  This can be useful
+     for kernels.  */
+  unsigned char entry_is_text_address;
+
   /* The value to pass to N_SET_FLAGS.  */
   unsigned char exec_hdr_flags;
 
@@ -561,8 +567,8 @@ NAME(aout,bfd_free_cached_info) PARAMS ((bfd *));
 #define NO_WRITE_HEADER_KLUDGE 0
 #endif
 
-#ifndef aout_32_bfd_is_local_label
-#define aout_32_bfd_is_local_label bfd_generic_is_local_label
+#ifndef aout_32_bfd_is_local_label_name
+#define aout_32_bfd_is_local_label_name bfd_generic_is_local_label_name
 #endif
 
 #ifndef WRITE_HEADERS
