@@ -1,6 +1,5 @@
 
-/*  A Bison parser, made from /home/work/misc/release/binutils-2.7.0.5/binutils/sysinfo.y
- by  Bison version A2.6 (Andrew Consortium)
+/*  A Bison parser, made from ./sysinfo.y with Bison version GNU Bison version 1.24
   */
 
 #define YYBISON 1  /* Identify Bison output.  */
@@ -12,7 +11,7 @@
 #define	NUMBER	262
 #define	UNIT	263
 
-#line 1 "/home/work/misc/release/binutils-2.7.0.5/binutils/sysinfo.y"
+#line 1 "./sysinfo.y"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -34,11 +33,28 @@ char *loop [] = {"","n","m","/*BAD*/"};
 char *names[] = {" ","[n]","[n][m]"};
 char *pnames[]= {"","*","**"};
 
-#line 24 "/home/work/misc/release/binutils-2.7.0.5/binutils/sysinfo.y"
+#line 24 "./sysinfo.y"
 typedef union {
  int i;
  char *s;
 } YYSTYPE;
+
+#ifndef YYLTYPE
+typedef
+  struct yyltype
+    {
+      int timestamp;
+      int first_line;
+      int first_column;
+      int last_line;
+      int last_column;
+      char *text;
+   }
+  yyltype;
+
+#define YYLTYPE yyltype
+#endif
+
 #ifndef YYDEBUG
 #define YYDEBUG 1
 #endif
@@ -115,15 +131,11 @@ static const short yyrline[] = { 0,
    157,   186,   204,   217,   230,   233,   338,   340,   343,   348,
    354,   356,   359,   360,   362,   363
 };
-#endif
-
-
-#if YYDEBUG != 0
 
 static const char * const yytname[] = {   "$","error","$undefined.","COND","REPEAT",
 "'('","')'","TYPE","NAME","NUMBER","UNIT","top","@1","it_list","it","@2","it_field_list",
 "repeat_it_field","@3","cond_it_field","@4","it_field","@5","attr_type","attr_desc",
-"attr_size","attr_id","enums","enum_list", NULL
+"attr_size","attr_id","enums","enum_list",""
 };
 #endif
 
@@ -185,7 +197,7 @@ static const short yycheck[] = {    11,
      0,     0,     8,     8,     6,    -1,     4
 };
 /* -*-C-*-  Note some compilers choke on comments on `#line' lines.  */
-#line 3 "/usr/lib/bison.simple"
+#line 3 "/usr/unsupported/share/bison.simple"
 
 /* Skeleton output parser for bison,
    Copyright (C) 1984, 1989, 1990 Free Software Foundation, Inc.
@@ -378,7 +390,7 @@ __yy_memcpy (char *from, char *to, int count)
 #endif
 #endif
 
-#line 192 "/usr/lib/bison.simple"
+#line 192 "/usr/unsupported/share/bison.simple"
 
 /* The user can define YYPARSE_PARAM as the name of an argument to be passed
    into yyparse.  The argument should have type void *.
@@ -677,7 +689,7 @@ yyreduce:
   switch (yyn) {
 
 case 1:
-#line 38 "/home/work/misc/release/binutils-2.7.0.5/binutils/sysinfo.y"
+#line 38 "./sysinfo.y"
 {
   switch (writecode)
     {
@@ -701,7 +713,7 @@ case 1:
  ;
     break;}
 case 2:
-#line 59 "/home/work/misc/release/binutils-2.7.0.5/binutils/sysinfo.y"
+#line 59 "./sysinfo.y"
 {
   switch (writecode) {
   case 'i':
@@ -716,7 +728,7 @@ case 2:
 ;
     break;}
 case 5:
-#line 81 "/home/work/misc/release/binutils-2.7.0.5/binutils/sysinfo.y"
+#line 81 "./sysinfo.y"
 {
 	it = yyvsp[-1].s; code = yyvsp[0].i;
 	switch (writecode) 
@@ -767,7 +779,7 @@ case 5:
       ;
     break;}
 case 6:
-#line 131 "/home/work/misc/release/binutils-2.7.0.5/binutils/sysinfo.y"
+#line 131 "./sysinfo.y"
 {
   switch (writecode) {
   case 'd': 
@@ -785,7 +797,7 @@ case 6:
 ;
     break;}
 case 11:
-#line 158 "/home/work/misc/release/binutils-2.7.0.5/binutils/sysinfo.y"
+#line 158 "./sysinfo.y"
 {
 	  rdepth++;
 	  switch (writecode) 
@@ -815,7 +827,7 @@ case 11:
 	;
     break;}
 case 12:
-#line 188 "/home/work/misc/release/binutils-2.7.0.5/binutils/sysinfo.y"
+#line 188 "./sysinfo.y"
 {
 	  repeat = oldrepeat;
 	  oldrepeat =0;
@@ -831,7 +843,7 @@ case 12:
 	;
     break;}
 case 13:
-#line 205 "/home/work/misc/release/binutils-2.7.0.5/binutils/sysinfo.y"
+#line 205 "./sysinfo.y"
 {
 	  switch (writecode) 
 	    {
@@ -845,7 +857,7 @@ case 13:
 	;
     break;}
 case 14:
-#line 218 "/home/work/misc/release/binutils-2.7.0.5/binutils/sysinfo.y"
+#line 218 "./sysinfo.y"
 {
 	  switch (writecode)
 	    {
@@ -858,11 +870,11 @@ case 14:
 	;
     break;}
 case 15:
-#line 232 "/home/work/misc/release/binutils-2.7.0.5/binutils/sysinfo.y"
+#line 232 "./sysinfo.y"
 {name = yyvsp[0].s; ;
     break;}
 case 16:
-#line 234 "/home/work/misc/release/binutils-2.7.0.5/binutils/sysinfo.y"
+#line 234 "./sysinfo.y"
 {
 	  char *desc = yyvsp[-8].s;
 	  char *type = yyvsp[-6].s;
@@ -965,31 +977,31 @@ char *ptr = pnames[rdepth];
 	;
     break;}
 case 17:
-#line 339 "/home/work/misc/release/binutils-2.7.0.5/binutils/sysinfo.y"
+#line 339 "./sysinfo.y"
 { yyval.s = yyvsp[0].s; ;
     break;}
 case 18:
-#line 340 "/home/work/misc/release/binutils-2.7.0.5/binutils/sysinfo.y"
+#line 340 "./sysinfo.y"
 { yyval.s = "INT";;
     break;}
 case 19:
-#line 345 "/home/work/misc/release/binutils-2.7.0.5/binutils/sysinfo.y"
+#line 345 "./sysinfo.y"
 { yyval.s = yyvsp[-1].s; ;
     break;}
 case 20:
-#line 350 "/home/work/misc/release/binutils-2.7.0.5/binutils/sysinfo.y"
+#line 350 "./sysinfo.y"
 { yyval.i = yyvsp[-1].i * yyvsp[0].i; ;
     break;}
 case 21:
-#line 355 "/home/work/misc/release/binutils-2.7.0.5/binutils/sysinfo.y"
+#line 355 "./sysinfo.y"
 { yyval.s = yyvsp[-1].s; ;
     break;}
 case 22:
-#line 356 "/home/work/misc/release/binutils-2.7.0.5/binutils/sysinfo.y"
+#line 356 "./sysinfo.y"
 { yyval.s = "dummy";;
     break;}
 case 26:
-#line 364 "/home/work/misc/release/binutils-2.7.0.5/binutils/sysinfo.y"
+#line 364 "./sysinfo.y"
 { 
 	  switch (writecode) 
 	    {
@@ -1003,7 +1015,7 @@ case 26:
     break;}
 }
    /* the action file gets copied in in place of this dollarsign */
-#line 487 "/usr/lib/bison.simple"
+#line 487 "/usr/unsupported/share/bison.simple"
 
   yyvsp -= yylen;
   yyssp -= yylen;
@@ -1199,7 +1211,7 @@ yyerrhandle:
   yystate = yyn;
   goto yynewstate;
 }
-#line 379 "/home/work/misc/release/binutils-2.7.0.5/binutils/sysinfo.y"
+#line 379 "./sysinfo.y"
 
 /* four modes
 
