@@ -41,6 +41,14 @@
 #include "elf/ppc.h"
 #endif
 
+static bfd_vma elf_s_get_size PARAMS ((symbolS *));
+static void elf_s_set_size PARAMS ((symbolS *, bfd_vma));
+static bfd_vma elf_s_get_align PARAMS ((symbolS *));
+static void elf_s_set_align PARAMS ((symbolS *, bfd_vma));
+static void elf_copy_symbol_attributes PARAMS ((symbolS *, symbolS *));
+static int elf_sec_sym_ok_for_reloc PARAMS ((asection *));
+static void adjust_stab_sections PARAMS ((bfd *, asection *, PTR));
+
 #ifdef NEED_ECOFF_DEBUG
 static boolean elf_get_extr PARAMS ((asymbol *, EXTR *));
 static void elf_set_index PARAMS ((asymbol *, bfd_size_type));
@@ -54,6 +62,7 @@ static void obj_elf_ident PARAMS ((int));
 static void obj_elf_weak PARAMS ((int));
 static void obj_elf_local PARAMS ((int));
 static void obj_elf_common PARAMS ((int));
+static void obj_elf_symver PARAMS ((int));
 static void obj_elf_data PARAMS ((int));
 static void obj_elf_text PARAMS ((int));
 
@@ -72,9 +81,12 @@ static const pseudo_typeS elf_pseudo_table[] =
   {"version", obj_elf_version, 0},
   {"weak", obj_elf_weak, 0},
 
-/* These are used for stabs-in-elf configurations.  */
+  /* These are used for stabs-in-elf configurations.  */
   {"line", obj_elf_line, 0},
 
+  /* This is a GNU extension to handle symbol versions.  */
+  {"symver", obj_elf_symver, 0},
+
   /* These are used for dwarf. */
   {"2byte", cons, 2},
   {"4byte", cons, 4},
@@ -260,7 +272,7 @@ obj_elf_common (ignore)
   *p = 0;
   symbolP = symbol_find_or_make (name);
   *p = c;
-  if (S_IS_DEFINED (symbolP))
+  if (S_IS_DEFINED (symbolP) && ! S_IS_COMMON (symbolP))
     {
       as_bad ("Ignoring attempt to re-define symbol");
       ignore_rest_of_line ();
@@ -322,12 +334,12 @@ obj_elf_common (ignore)
 	  record_alignment (bss_section, align);
 	  subseg_set (bss_section, 0);
 	  if (align)
-	    frag_align (align, 0);
+	    frag_align (align, 0, 0);
 	  if (S_GET_SEGMENT (symbolP) == bss_section)
 	    symbolP->sy_frag->fr_symbol = 0;
 	  symbolP->sy_frag = frag_now;
-	  pfrag = frag_var (rs_org, 1, 1, (relax_substateT) 0, symbolP, size,
-			    (char *) 0);
+	  pfrag = frag_var (rs_org, 1, 1, (relax_substateT) 0, symbolP,
+			    (offsetT) size, (char *) 0);
 	  *pfrag = 0;
 	  S_SET_SIZE (symbolP, size);
 	  S_SET_SEGMENT (symbolP, bss_section);
@@ -340,8 +352,7 @@ obj_elf_common (ignore)
 	  S_SET_VALUE (symbolP, (valueT) size);
 	  S_SET_ALIGN (symbolP, temp);
 	  S_SET_EXTERNAL (symbolP);
-	  /* should be common, but this is how gas does it for now */
-	  S_SET_SEGMENT (symbolP, bfd_und_section_ptr);
+	  S_SET_SEGMENT (symbolP, bfd_com_section_ptr);
 	}
     }
   else
@@ -363,6 +374,9 @@ obj_elf_common (ignore)
 	;
       goto allocate_common;
     }
+
+  symbolP->bsym->flags |= BSF_OBJECT;
+
   demand_empty_rest_of_line ();
   return;
 
@@ -439,8 +453,8 @@ obj_elf_weak (ignore)
   demand_empty_rest_of_line ();
 }
 
-segT previous_section;
-int previous_subsection;
+static segT previous_section;
+static int previous_subsection;
 
 /* Handle the .section pseudo-op.  This code supports two different
    syntaxes.
@@ -741,7 +755,16 @@ obj_elf_section (xxx)
 	    as_warn ("Setting incorrect section type for %s", string);
 
 	  if ((attr &~ special_sections[i].attributes) != 0)
-	    as_warn ("Setting incorrect section attributes for %s", string);
+	    {
+	      /* As a GNU extension, we permit a .note section to be
+                 allocatable.  If the linker sees an allocateable
+                 .note section, it will create a PT_NOTE segment in
+                 the output file.  */
+	      if (strcmp (string, ".note") != 0
+		  || attr != SHF_ALLOC)
+		as_warn ("Setting incorrect section attributes for %s",
+			 string);
+	    }
 	  attr |= special_sections[i].attributes;
 
 	  break;
@@ -790,9 +813,17 @@ static void
 obj_elf_data (i)
      int i;
 {
+#ifdef md_flush_pending_output
+  md_flush_pending_output ();
+#endif
+
   previous_section = now_seg;
   previous_subsection = now_subseg;
   s_data (i);
+
+#ifdef md_elf_section_change_hook
+  md_elf_section_change_hook ();
+#endif
 }
 
 /* Change to the .text section.  */
@@ -801,9 +832,27 @@ static void
 obj_elf_text (i)
      int i;
 {
+#ifdef md_flush_pending_output
+  md_flush_pending_output ();
+#endif
+
   previous_section = now_seg;
   previous_subsection = now_subseg;
   s_text (i);
+
+#ifdef md_elf_section_change_hook
+  md_elf_section_change_hook ();
+#endif
+}
+
+/* This can be called from the processor backends if they change
+   sections.  */
+
+void
+obj_elf_section_change_hook ()
+{
+  previous_section = now_seg;
+  previous_subsection = now_subseg;
 }
 
 void
@@ -815,8 +864,17 @@ obj_elf_previous (ignore)
       as_bad (".previous without corresponding .section; ignored");
       return;
     }
+
+#ifdef md_flush_pending_output
+  md_flush_pending_output ();
+#endif
+
   subseg_set (previous_section, previous_subsection);
   previous_section = 0;
+
+#ifdef md_elf_section_change_hook
+  md_elf_section_change_hook ();
+#endif
 }
 
 static void
@@ -829,6 +887,68 @@ obj_elf_line (ignore)
   demand_empty_rest_of_line ();
 }
 
+/* This handle the .symver pseudo-op, which is used to specify a
+   symbol version.  The syntax is ``.symver NAME,SYMVERNAME''.
+   SYMVERNAME may contain ELF_VER_CHR ('@') characters.  This
+   pseudo-op causes the assembler to emit a symbol named SYMVERNAME
+   with the same value as the symbol NAME.  */
+
+static void
+obj_elf_symver (ignore)
+     int ignore;
+{
+  char *name;
+  char c;
+  symbolS *sym;
+
+  name = input_line_pointer;
+  c = get_symbol_end ();
+
+  sym = symbol_find_or_make (name);
+
+  *input_line_pointer = c;
+
+  if (sym->sy_obj.versioned_name != NULL)
+    {
+      as_bad ("multiple .symver directives for symbol `%s'",
+	      S_GET_NAME (sym));
+      ignore_rest_of_line ();
+      return;
+    }
+
+  SKIP_WHITESPACE ();
+  if (*input_line_pointer != ',')
+    {
+      as_bad ("expected comma after name in .symver");
+      ignore_rest_of_line ();
+      return;
+    }
+
+  ++input_line_pointer;
+  name = input_line_pointer;
+  while (1)
+    {
+      c = get_symbol_end ();
+      if (c != ELF_VER_CHR)
+	break;
+      *input_line_pointer++ = c;
+    }
+
+  sym->sy_obj.versioned_name = xstrdup (name);
+
+  *input_line_pointer = c;
+
+  if (strchr (sym->sy_obj.versioned_name, ELF_VER_CHR) == NULL)
+    {
+      as_bad ("missing version name in `%s' for symbol `%s'",
+	      sym->sy_obj.versioned_name, S_GET_NAME (sym));
+      ignore_rest_of_line ();
+      return;
+    }
+
+  demand_empty_rest_of_line ();
+}
+
 void
 obj_read_begin_hook ()
 {
@@ -842,7 +962,8 @@ void
 obj_symbol_new_hook (symbolP)
      symbolS *symbolP;
 {
-  symbolP->sy_obj = 0;
+  symbolP->sy_obj.size = NULL;
+  symbolP->sy_obj.versioned_name = NULL;
 
 #ifdef NEED_ECOFF_DEBUG
   if (ECOFF_DEBUGGING)
@@ -906,7 +1027,7 @@ obj_elf_version (ignore)
 	    FRAG_APPEND_1_CHAR (ch);
 	  }
 	}
-      frag_align (2, 0);
+      frag_align (2, 0, 0);
 
       subseg_set (seg, subseg);
     }
@@ -953,8 +1074,8 @@ obj_elf_size (ignore)
     S_SET_SIZE (sym, exp.X_add_number);
   else
     {
-      sym->sy_obj = (expressionS *) xmalloc (sizeof (expressionS));
-      *sym->sy_obj = exp;
+      sym->sy_obj.size = (expressionS *) xmalloc (sizeof (expressionS));
+      *sym->sy_obj.size = exp;
     }
   demand_empty_rest_of_line ();
 }
@@ -1156,27 +1277,90 @@ elf_frob_symbol (symp, puntp)
     ecoff_frob_symbol (symp);
 #endif
 
-  if (symp->sy_obj)
+  if (symp->sy_obj.size != NULL)
     {
-      switch (symp->sy_obj->X_op)
+      switch (symp->sy_obj.size->X_op)
 	{
 	case O_subtract:
 	  S_SET_SIZE (symp,
-		      (S_GET_VALUE (symp->sy_obj->X_add_symbol)
-		       + symp->sy_obj->X_add_number
-		       - S_GET_VALUE (symp->sy_obj->X_op_symbol)));
+		      (S_GET_VALUE (symp->sy_obj.size->X_add_symbol)
+		       + symp->sy_obj.size->X_add_number
+		       - S_GET_VALUE (symp->sy_obj.size->X_op_symbol)));
 	  break;
 	case O_constant:
 	  S_SET_SIZE (symp,
-		      (S_GET_VALUE (symp->sy_obj->X_add_symbol)
-		       + symp->sy_obj->X_add_number));
+		      (S_GET_VALUE (symp->sy_obj.size->X_add_symbol)
+		       + symp->sy_obj.size->X_add_number));
 	  break;
 	default:
 	  as_bad (".size expression too complicated to fix up");
 	  break;
 	}
-      free (symp->sy_obj);
-      symp->sy_obj = 0;
+      free (symp->sy_obj.size);
+      symp->sy_obj.size = NULL;
+    }
+
+  if (symp->sy_obj.versioned_name != NULL)
+    {
+      /* This symbol was given a new name with the .symver directive.
+
+         If this is an external reference, just rename the symbol to
+         include the version string.  This will make the relocs be
+         against the correct versioned symbol.
+
+	 If this is a definition, add an alias.  FIXME: Using an alias
+	 will permit the debugging information to refer to the right
+	 symbol.  However, it's not clear whether it is the best
+	 approach.  */
+
+      if (! S_IS_DEFINED (symp))
+	{
+	  char *p;
+
+	  /* Verify that the name isn't using the @@ syntax--this is
+             reserved for definitions of the default version to link
+             against.  */
+	  p = strchr (symp->sy_obj.versioned_name, ELF_VER_CHR);
+	  know (p != NULL);
+	  if (p[1] == ELF_VER_CHR)
+	    {
+	      as_bad ("invalid attempt to declare external version name as default in symbol `%s'",
+		      symp->sy_obj.versioned_name);
+	      *puntp = true;
+	    }
+	  S_SET_NAME (symp, symp->sy_obj.versioned_name);
+	}
+      else
+	{
+	  symbolS *symp2;
+
+	  /* FIXME: Creating a new symbol here is risky.  We're in the
+             final loop over the symbol table.  We can get away with
+             it only because the symbol goes to the end of the list,
+             where the loop will still see it.  It would probably be
+             better to do this in obj_frob_file_before_adjust. */
+
+	  symp2 = symbol_find_or_make (symp->sy_obj.versioned_name);
+
+	  /* Now we act as though we saw symp2 = sym.  */
+
+	  S_SET_SEGMENT (symp2, S_GET_SEGMENT (symp));
+
+	  /* Subtracting out the frag address here is a hack because
+             we are in the middle of the final loop.  */
+	  S_SET_VALUE (symp2, S_GET_VALUE (symp) - symp->sy_frag->fr_address);
+
+	  symp2->sy_frag = symp->sy_frag;
+
+	  /* This will copy over the size information.  */
+	  copy_symbol_attributes (symp2, symp);
+
+	  if (S_IS_WEAK (symp))
+	    S_SET_WEAK (symp2);
+
+	  if (S_IS_EXTERNAL (symp))
+	    S_SET_EXTERNAL (symp2);
+	}
     }
 
   /* Double check weak symbols.  */
@@ -1188,10 +1372,11 @@ elf_frob_symbol (symp, puntp)
     }
 
 #ifdef TC_MIPS
-  /* The Irix 5 assembler appears to set the type of any common symbol
-     to STT_OBJECT.  We try to be compatible, since the Irix 5 linker
-     apparently sometimes cares.  FIXME: What about Irix 6?  */
-  if (S_IS_COMMON (symp))
+  /* The Irix 5 and 6 assemblers set the type of any common symbol and
+     any undefined non-function symbol to STT_OBJECT.  We try to be compatible,
+     since newer Irix 5 and 6 linkers care.  */
+  if (S_IS_COMMON (symp)
+      || (! S_IS_DEFINED (symp) && ((symp->bsym->flags & BSF_FUNCTION) == 0)))
     symp->bsym->flags |= BSF_OBJECT;
 #endif
 
@@ -1374,7 +1559,7 @@ sco_id ()
   p = frag_more (4);
   md_number_to_chars (p, 0x0000, 4);
  
-  frag_align (2, 0); 
+  frag_align (2, 0, 0);
 
   /* We probably can't restore the current segment, for there likely
      isn't one yet...  */
