@@ -15,7 +15,7 @@
 
    You should have received a copy of the GNU General Public
    License along with GAS; see the file COPYING.  If not, write
-   to the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+   to the Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
    Written by the Center for Software Science at the University of Utah
    and by Cygnus Support.  */
@@ -217,24 +217,6 @@ adjust_stab_sections (abfd, sec, xxx)
   bfd_h_put_32 (abfd, (bfd_vma) strsz, (bfd_byte *) p + 8);
 }
 
-#if 0
-/* Adjust the VMA address for each $CODE$ subspace.  */
-static void
-adjust_code_sections (abfd, sec, xxx)
-     bfd *abfd;
-     asection *sec;
-     PTR xxx;
-{
-  static unsigned size_so_far = 0;
-
-  if (strcmp (sec->name, "$CODE$"))
-    return;
-
-  bfd_set_section_vma (stdoutput, sec, size_so_far);
-  size_so_far += bfd_get_section_size_before_reloc (sec);
-}
-#endif
-
 /* Called late in the asssembly phase to adjust the special
    stab entry and to set the starting address for each code subspace.  */
 
@@ -242,8 +224,4 @@ void
 som_frob_file ()
 {
   bfd_map_over_sections (stdoutput, adjust_stab_sections, (PTR) 0);
-#if 0
-  See comment in tc-hppa.c:pa_proc about GDB lossage
-  bfd_map_over_sections (stdoutput, adjust_code_sections, (PTR) 0);
-#endif
 }
