@@ -16,38 +16,39 @@
  * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
  * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
+#include "hertz.h"
 
-#ifndef lint
-static char sccsid[] = "@(#)hertz.c	5.4 (Berkeley) 6/1/90";
-#endif /* not lint */
-
-#include <sys/time.h>
-
-    /*
-     *	discover the tick frequency of the machine
-     *	if something goes wrong, we return 0, an impossible hertz.
-     */
-#define	HZ_WRONG	0
 
 #ifdef __MSDOS__
 #define HERTZ 18
 #endif
 
-hertz()
+int
+hertz ()
 {
 #ifdef HERTZ
-	return HERTZ;
-#else
-	struct itimerval tim;
+  return HERTZ;
+#else /* ! defined (HERTZ) */
+#ifdef HAVE_SETITIMER
+  struct itimerval tim;
 
-	tim.it_interval.tv_sec = 0;
-	tim.it_interval.tv_usec = 1;
-	tim.it_value.tv_sec = 0;
-	tim.it_value.tv_usec = 0;
-	setitimer(ITIMER_REAL, &tim, 0);
-	setitimer(ITIMER_REAL, 0, &tim);
-	if (tim.it_interval.tv_usec < 2)
-		return(HZ_WRONG);
-	return (1000000 / tim.it_interval.tv_usec);
-#endif
+  tim.it_interval.tv_sec = 0;
+  tim.it_interval.tv_usec = 1;
+  tim.it_value.tv_sec = 0;
+  tim.it_value.tv_usec = 0;
+  setitimer (ITIMER_REAL, &tim, 0);
+  setitimer (ITIMER_REAL, 0, &tim);
+  if (tim.it_interval.tv_usec < 2)
+    {
+      return HZ_WRONG;
+    }
+  return 1000000 / tim.it_interval.tv_usec;
+#else /* ! defined (HAVE_SETITIMER) */
+#if defined (HAVE_SYSCONF) && defined (_SC_CLK_TCK)
+  return sysconf (_SC_CLK_TCK);
+#else /* ! defined (HAVE_SYSCONF) || ! defined (_SC_CLK_TCK) */
+  return HZ_WRONG;
+#endif /* ! defined (HAVE_SYSCONF) || ! defined (_SC_CLK_TCK) */
+#endif /* ! defined (HAVE_SETITIMER) */
+#endif /* ! defined (HERTZ) */
 }
