@@ -16,8 +16,7 @@ target=$4
 target_alias=$5
 DEFAULT_EMULATION=$6
 NATIVE_LIB_DIRS=$7
-LIB_PATH=$8
-EMULATION_NAME=$9
+EMULATION_NAME=$8
 tool_lib=`echo ${libdir} | sed -e 's|/lib$||'`/${9-$target_alias}/lib
 
 # Include the emulation-specific parameters:
@@ -61,13 +60,7 @@ if [ "x${LIB_PATH}" = "x" ] ; then
 fi
 
 # Always search $(tooldir)/lib, aka /usr/local/TARGET/lib.
-if [ $DEFAULT_EMULATION = $EMULATION_NAME ]; then
-  LIB_PATH=${LIB_PATH}:${tool_lib}
-else
-  # This is not correct one since this is for a different target.
-  # LIB_PATH=${LIB_PATH}:${tool_lib}
-  LIB_PATH=${tool_lib}
-fi
+LIB_PATH=${LIB_PATH}:${tool_lib}
 
 LIB_SEARCH_DIRS=`echo ${LIB_PATH} | tr ':' ' ' | sed -e 's/\([^ ][^ ]*\)/SEARCH_DIR(\1);/g'`
 
@@ -136,5 +129,5 @@ fi
 
 test "$DEFAULT_EMULATION" = "$EMULATION_NAME" && COMPILE_IN=true
 
-# Generate em_${EMULATION_NAME}.c.
+# Generate e${EMULATION_NAME}.c.
 . ${srcdir}/emultempl/${TEMPLATE_NAME-generic}.em
