@@ -1,5 +1,5 @@
 /* ldemul.c -- clearing house for ld emulation states
-   Copyright (C) 1991, 1993 Free Software Foundation, Inc.
+   Copyright (C) 1991, 92, 93, 94, 95, 96, 1997 Free Software Foundation, Inc.
 
 This file is part of GLD, the Gnu Linker.
 
@@ -145,6 +145,17 @@ ldemul_parse_args (argc, argv)
   return 0;
 }
 
+/* Let the emulation code handle an unrecognized file.  */
+
+boolean
+ldemul_unrecognized_file (entry)
+     lang_input_statement_type *entry;
+{
+  if (ld_emulation->unrecognized_file)
+    return (*ld_emulation->unrecognized_file) (entry);
+  return false;
+}
+
 char *
 ldemul_choose_target()
 {
@@ -156,7 +167,7 @@ ldemul_choose_target()
 char *
 ldemul_default_target()
 {
-  char *from_outside = (char *) getenv(TARGET_ENVIRON);
+  char *from_outside = getenv (TARGET_ENVIRON);
   if (from_outside != (char *)NULL)
     return from_outside;
   return ld_emulation->target_name;
