@@ -99,6 +99,7 @@ exp_print_token (code)
 	{ NEXT,"NEXT" },
 	{ SIZEOF,"SIZEOF" },
 	{ ADDR,"ADDR" },
+	{ LOADADDR,"LOADADDR" },
 	{ MEMORY,"MEMORY" },
 	{ DEFINED,"DEFINED" },
 	{ TARGET_K,"TARGET" },
@@ -294,6 +295,16 @@ fold_binary (tree, current_section, allocation_done, dot, dotp)
 	      BOP(ANDAND,&&);
 	      BOP(OROR,||);
 
+	    case MAX:
+	      if (result.value < other.value)
+		result = other;
+	      break;
+
+	    case MIN:
+	      if (result.value > other.value)
+		result = other;
+	      break;
+
 	    default:
 	      FAIL();
 	    }
@@ -410,6 +421,25 @@ fold_name (tree, current_section, allocation_done, dot)
 	else
 	  result = invalid ();
 	break;
+
+      case LOADADDR:
+	if (allocation_done != lang_first_phase_enum)
+	  {
+	    lang_output_section_statement_type *os;
+
+	    os = lang_output_section_find (tree->name.name);
+	    check (os, tree->name.name, "LOADADDR");
+	    if (os->load_base == NULL)
+	      result = new_rel (0, os);
+	    else
+	      result = exp_fold_tree_no_dot (os->load_base,
+					     abs_output_section,
+					     allocation_done);
+	  }
+	else
+	  result = invalid ();
+	break;
+
       case SIZEOF:
 	if (allocation_done != lang_first_phase_enum)
 	  {
