@@ -1,5 +1,5 @@
 /* ldmain.h -
-   Copyright 1991, 1992 Free Software Foundation, Inc.
+   Copyright 1991, 1992, 1993 Free Software Foundation, Inc.
 
    This file is part of GLD, the Gnu Linker.
 
@@ -15,8 +15,24 @@
 
    You should have received a copy of the GNU General Public License
    along with GLD; see the file COPYING.  If not, write to
-   the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   the Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
-void enter_global_ref PARAMS ((asymbol **, CONST char *));
-void ldmain_open_file_read_symbol PARAMS ((
-		struct lang_input_statement_struct *));
+#ifndef LDMAIN_H
+#define LDMAIN_H
+
+extern char *program_name;
+extern bfd *output_bfd;
+extern char *default_target;
+extern boolean trace_files;
+extern boolean trace_file_tries;
+extern boolean version_printed;
+extern boolean whole_archive;
+extern int g_switch_value;
+extern const char *output_filename;
+extern struct bfd_link_info link_info;
+
+extern void add_ysym PARAMS ((const char *));
+extern void add_wrap PARAMS ((const char *));
+extern void add_keepsyms_file PARAMS ((const char *filename));
+
+#endif
