@@ -1,5 +1,5 @@
 /* mips16-opc.c.  Mips16 opcode table.
-   Copyright 1996 Free Software Foundation, Inc.
+   Copyright 1996, 1997 Free Software Foundation, Inc.
    Contributed by Ian Lance Taylor, Cygnus Support
 
 This file is part of GDB, GAS, and the GNU binutils.
@@ -128,6 +128,7 @@ const struct mips_opcode mips16_opcodes[] = {
 {"div",     "z,v,y",	0, (int) M_DIV_3, INSN_MACRO },
 {"divu",    "0,x,y",	0xe81b, 0xf81f, RD_x|RD_y|WR_HI|WR_LO },
 {"divu",    "z,v,y",	0, (int) M_DIVU_3, INSN_MACRO },
+{"dmul",    "z,v,y",	3, (int) M_DMUL, INSN_MACRO },
 {"dmult",   "x,y",	0xe81c, 0xf81f, RD_x|RD_y|WR_HI|WR_LO|I3 },
 {"dmultu",  "x,y",	0xe81d, 0xf81f, RD_x|RD_y|WR_HI|WR_LO|I3 },
 {"drem",    "0,x,y",	0xe81e, 0xf81f, RD_x|RD_y|WR_HI|WR_LO|I3 },
@@ -146,6 +147,8 @@ const struct mips_opcode mips16_opcodes[] = {
 {"dsubu",   "z,v,y",	0xe002, 0xf803, WR_z|RD_x|RD_y|I3 },
 {"dsubu",   "y,x,4",	0, (int) M_DSUBU_I, INSN_MACRO },
 {"dsubu",   "y,j",	0, (int) M_DSUBU_I_2, INSN_MACRO },
+{"exit",    "L",	0xed09, 0xff1f, TRAP },
+{"exit",    "L",	0xee09, 0xff1f, TRAP },
 {"exit",    "L",	0xef09, 0xff1f, TRAP },
 {"entry",   "l",	0xe809, 0xf81f, TRAP },
 {"extend",  "e",	0xf000, 0xf800, 0 },
@@ -177,6 +180,7 @@ const struct mips_opcode mips16_opcodes[] = {
 {"mflo",    "x",	0xe812, 0xf8ff, WR_x|RD_LO },
 {"move",    "y,X",	0x6700, 0xff00, WR_y|RD_X },
 {"move",    "Y,Z",	0x6500, 0xff00, WR_Y|RD_Z },
+{"mul",     "z,v,y",	0, (int) M_MUL, INSN_MACRO },
 {"mult",    "x,y",	0xe818, 0xf81f, RD_x|RD_y|WR_HI|WR_LO },
 {"multu",   "x,y",	0xe819, 0xf81f, RD_x|RD_y|WR_HI|WR_LO },
 {"neg",	    "x,w",	0xe80b, 0xf81f, WR_x|RD_y },
